package weaver.interfaces.kqsolution.crob;

import com.engine.kqsolution.entity.HistoryResourcePO;
import com.engine.kqsolution.util.ResourceSnipUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author liang.cheng
 * @Date 2024/8/28 9:12 AM
 * @Description: 花名册快照定时任务
 * @Version 1.0
 */
public class ResourceSnipCrob extends BaseCronJob {


    @Override
    public void execute() {
        BaseBean bb = new BaseBean();
        RecordSet rs = new RecordSet();

        String account = bb.getPropValue("jcsecond", "account");
        String bankField = bb.getPropValue("jcsecond", "bankField");
        String regresidentplace = bb.getPropValue("jcsecond", "regresidentplace");
        String femdate = bb.getPropValue("jcsecond", "femdate");

        //1.获取数据
        List<HistoryResourcePO> snipList = new ArrayList<>();
        rs.executeQuery("select a.id,a.workcode,a.lastname,a.birthday,a.sex,a.subcompanyid1,a.departmentid,a.managerid,a.companystartdate,\n" +
                "a.createdate,a.accumfundaccount,b."+account+" as account,b."+bankField+" as bankname,c."+femdate+" as femdate,\n" +
                "a.startdate,a.enddate,a.jobtitle,a.status,a.educationlevel,a.mobile,a.certificatenum,\n" +
                "a.nativeplace,b."+regresidentplace+" as regresidentplace from hrmresource a\n" +
                "left join cus_fielddata b on a.id = b.id and b.scopeid = 1\n" +
                "left join cus_fielddata c on a.id = c.id and c.scopeid = -1\n" +
                "where a.status < 4");

        while (rs.next()) {
            HistoryResourcePO build = HistoryResourcePO.builder()
                    .resourceId(Util.getIntValue(rs.getString("id")))
                    .workcode(Util.null2String(rs.getString("workcode")))
                    .lastName(Util.null2String(rs.getString("lastname")))
                    .birthday(Util.null2String(rs.getString("birthday")))
                    .sex("0".equals(Util.null2String(rs.getString("sex"))) ? "男" : "女")
                    .subCompany(ResourceSnipUtils.selectSubCompanyName(Util.null2String(rs.getString("subcompanyid1"))))
                    .department(ResourceSnipUtils.selectDeptName(Util.null2String(rs.getString("departmentid"))))
                    .managerId(ResourceSnipUtils.selectResourceName(Util.null2String(rs.getString("managerid"))))
                    .companyStartDate(Util.null2String(rs.getString("companystartdate")))
                    .createDate(Util.null2String(rs.getString("createdate")))
                    .accumfundAccount(Util.null2String(rs.getString("accumfundaccount")))
                    .accountId(Util.null2String(rs.getString("account")))
                    .bankName(ResourceSnipUtils.selectBankName(Util.null2String(rs.getString("bankname")),bankField))
                    .firstEnrollmentDate(Util.null2String(rs.getString("femdate")))
                    .startdate(Util.null2String(rs.getString("startdate")))
                    .endDate(Util.null2String(rs.getString("enddate")))
                    .jobTitle(ResourceSnipUtils.selectJobName(Util.null2String(rs.getString("jobtitle"))))
                    .status(Util.null2String(rs.getString("status")))
                    .educationlevel(ResourceSnipUtils.selectEducName(Util.null2String(rs.getString("educationlevel"))))
                    .mobile(Util.null2String(rs.getString("mobile")))
                    .certificatenum(Util.null2String(rs.getString("certificatenum")))
                    .nativeplace(Util.null2String(rs.getString("nativeplace")))
                    .regresidentplace(Util.null2String(rs.getString("regresidentplace")))
                    .build();
            snipList.add(build);
        }

        //获取当前月格式2024-08
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
        String currentMonth = LocalDate.now().format(formatter);
        String modeId = bb.getPropValue("jcsecond", "modeId");
        String currentDate = TimeUtil.getCurrentDateString();
        String onlyCurrentTime = TimeUtil.getOnlyCurrentTimeString();

        //2.存储快照
        //区分是否数据存在，List存在部分更新，未存在数据新增
        List<Integer> hList = new ArrayList<>();
        rs.executeQuery("select ryid from uf_hmckz where  cdrq = ?",currentMonth);
        while (rs.next()) {
            hList.add(Util.getIntValue(rs.getString("ryid")));
        }

        //已存在数据
        List<HistoryResourcePO> uList = snipList.stream()
                .filter(item -> hList.contains(item.getResourceId()))
                .collect(Collectors.toList());
        //未存在数据
        List<HistoryResourcePO> iList = snipList.stream()
                .filter(item -> !hList.contains(item.getResourceId()))
                .collect(Collectors.toList());

        for (HistoryResourcePO i : uList) {
            rs.executeUpdate("update uf_hmckz set gh = ?,xm = ?,csrq = ?,xb = ?,fb = ?,bm = ?,zjsj = ?,rzrq = ?,cjrq = ?,gjjzh = ?," +
                    " gzzh = ?,gzyx = ?, sccbsj = ?,htksrq = ?,htjsrq = ?,gw = ?,zt = ?,xl = ?,dh = ?,sfzh = ?,jg = ?," +
                            " hkszd = ?, lzrq = ? where id = ? and cdrq = ?",i.getWorkcode(),i.getLastName(),i.getBirthday(),
                    i.getSex(),i.getSubCompany(),i.getDepartment(), i.getManagerId(),i.getCompanyStartDate(),i.getCreateDate(),i.getAccumfundAccount(),
                    i.getAccountId(),i.getBankName(),i.getFirstEnrollmentDate(),i.getStartdate(),i.getEndDate(),i.getJobTitle(),i.getStatus(),i.getEducationlevel(),
                    i.getMobile(),i.getCertificatenum(),i.getNativeplace(),i.getRegresidentplace(),i.getChangeDate(),i.getResourceId(),currentMonth);
        }

        for (HistoryResourcePO i : iList) {
            rs.executeUpdate("insert into uf_hmckz(ryid,gh,xm,csrq,xb,fb,bm,zjsj,rzrq,cjrq,gjjzh,gzzh,gzyx,sccbsj,htksrq,htjsrq,gw,zt,xl,dh,sfzh,jg,hkszd,cdrq,lzrq,formmodeid,modedatacreatedate,modedatacreatetime) " +
                    " values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",i.getResourceId(),i.getWorkcode(),i.getLastName(),i.getBirthday(),
                    i.getSex(),i.getSubCompany(),i.getDepartment(), i.getManagerId(),i.getCompanyStartDate(),i.getCreateDate(),i.getAccumfundAccount(),
                    i.getAccountId(),i.getBankName(),i.getFirstEnrollmentDate(),i.getStartdate(),i.getEndDate(),i.getJobTitle(),i.getStatus(),i.getEducationlevel(),
                    i.getMobile(),i.getCertificatenum(),i.getNativeplace(),i.getRegresidentplace(),currentMonth,i.getChangeDate(),modeId,currentDate,onlyCurrentTime);
        }

    }
}
